@@ -16,11 +16,16 @@ except ImportError:
     asyncio_support = False
 
 
+def pytest_configure(config):
+    config.addinivalue_line("markers", "dont_patch_engines: do not patch sqla engines")
+
+
 @fixture(scope="session")
 def db_host():
     """Default db host used by depending fixtures.
 
-    When CI key is set in environment variables, it uses `postgres` as host name else, host used is `localhost`
+    When CI key is set in environment variables, it uses `postgres` as host name else,
+    host used is `localhost`
     """
 
     return "postgres" if "CI" in os.environ else "localhost"
@@ -49,9 +54,8 @@ def db_url(db_host, db_user):
 @fixture(scope="session")
 def sqla_connection(db_url):
     engine = create_engine(db_url)
-    connection = engine.connect()
-    yield connection
-    connection.close()
+    with engine.connect() as connection:
+        yield connection
 
 
 @fixture(scope="session")
@@ -69,7 +73,10 @@ def db_migration(db_url, sqla_connection, alembic_ini_path):
     alembic_config = Config(file_=alembic_ini_path)
     alembic_config.set_main_option("sqlalchemy.url", db_url)
 
-    sqla_connection.execute(text("DROP SCHEMA public CASCADE; CREATE SCHEMA public;"))
+    with sqla_connection.begin():
+        sqla_connection.execute(
+            text("DROP SCHEMA public CASCADE; CREATE SCHEMA public;")
+        )
 
     command.upgrade(alembic_config, "head")
     yield
@@ -84,7 +91,7 @@ def sqla_modules():
     )
 
 
-@fixture(autouse=True)
+@fixture
 def sqla_reflection(sqla_modules, sqla_connection, db_url):
     import fastapi_sqla
 
@@ -92,15 +99,18 @@ def sqla_reflection(sqla_modules, sqla_connection, db_url):
     fastapi_sqla.Base.prepare(sqla_connection)
 
 
-@fixture(autouse=True)
-def engine_from_config(request, db_url, sqla_connection, sqla_transaction):
+@fixture
+def patch_engine_from_config(request, db_url, sqla_connection, sqla_transaction):
     """So that all DB operations are never written to db for real."""
+    from fastapi_sqla.sqla import _Session
+
     if "dont_patch_engines" in request.keywords:
         yield
 
     else:
-        with patch("fastapi_sqla.engine_from_config") as engine_from_config:
+        with patch("fastapi_sqla.sqla.engine_from_config") as engine_from_config:
             engine_from_config.return_value = sqla_connection
+            _Session.configure(bind=sqla_connection)
             yield engine_from_config
 
 
@@ -112,17 +122,17 @@ def sqla_transaction(sqla_connection):
 
 
 @fixture
-def session(sqla_transaction, sqla_connection):
+def session(
+    sqla_transaction, sqla_connection, sqla_reflection, patch_engine_from_config
+):
     """Sqla session to use when creating db fixtures.
 
     While it does not write any record in DB, the application will still be able to
     access any record committed with that session.
     """
-    import fastapi_sqla
+    import fastapi_sqla.sqla
 
-    session = fastapi_sqla._Session(bind=sqla_connection)
-    yield session
-    session.close()
+    yield fastapi_sqla.sqla._Session(bind=sqla_connection)
 
 
 def format_async_async_sqlalchemy_url(url):
@@ -151,25 +161,32 @@ if asyncio_support:
             yield connection
             await connection.rollback()
 
-    @fixture(autouse=True)
-    async def patch_async_sessionmaker(
-        async_sqlalchemy_url, async_sqla_connection, request
-    ):
+    @fixture
+    async def patch_new_engine(async_sqlalchemy_url, async_sqla_connection, request):
         """So that all async DB operations are never written to db for real."""
+        from fastapi_sqla.asyncio_support import _AsyncSession
+
         if "dont_patch_engines" in request.keywords:
             yield
 
         else:
-            with patch(
-                "fastapi_sqla.asyncio_support.create_async_engine"
-            ) as create_async_engine:
-                create_async_engine.return_value = async_sqla_connection
-                yield create_async_engine
+            with patch("fastapi_sqla.asyncio_support.new_engine") as new_engine:
+                new_engine.return_value = async_sqla_connection
+                _AsyncSession.configure(
+                    bind=async_sqla_connection, expire_on_commit=False
+                )
+                yield new_engine
 
     @fixture
-    async def async_session(async_sqla_connection):
+    async def async_session(async_sqla_connection, sqla_reflection, patch_new_engine):
         from fastapi_sqla.asyncio_support import _AsyncSession
 
         session = _AsyncSession(bind=async_sqla_connection)
         yield session
         await session.close()
+
+else:
+
+    @fixture
+    async def patch_new_engine():
+        pass
